﻿using System.Collections.Generic;
using System;
using System.IO;
using SqlBI.DtExec.Classes;

namespace SqlBI.DtExec.TestApplication {
    /// <summary>
    /// 
    ///     This simple test application uses the ClientAgent to communicate via WCF
    ///     with the server (which may be hosted by a service, IIS, command line or
    ///     whatever means we need to use).
    ///     
    ///     The configuration of the WCF endpoint is made entirely in the app.config file
    ///     so that it can be changed without any need to recompile the project.
    /// 
    /// </summary>
    class Program {
        private static List<StreamWriter> LogStream = new List<StreamWriter> ();
        private static List<StreamWriter> EventStream = new List<StreamWriter> ();
        private static Configuration Config = new Configuration ();

        /// <summary>
        /// 
        ///     Entry point of the program
        /// 
        /// </summary>
        /// <param name="args"></param>
        static void Main (string[] args) {
            LoadConfiguration (args[0]);

            foreach (PackageInformation package in Config.Packages) {
                if (package.Execute) {
                    RunPackage (Config, package);
                }
            }
            //Console.WriteLine ("Press ENTER to continue...");
            //Console.ReadLine ();
        }

        /// <summary>
        /// 
        ///     Loads the configuration from the XML file
        /// 
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        private static void LoadConfiguration (string fileName) {
            ObjectXMLSerializer o = new ObjectXMLSerializer ();
            Config = (Configuration) o.Load (Config, fileName);
            Config.ApplyDefaultRules ();
        }

        /// <summary>
        /// 
        ///     When we receive a message, we send it to the 
        ///     correct stream, depending on the type of the
        ///     message
        /// 
        /// </summary>
        /// <param name="message"></param>
        static public void LogMessage (LogItem logItem) {
            switch (logItem.LogType) {
                case LogType.Log:
                    foreach (StreamWriter s in LogStream) {
                        s.WriteLine (logItem.TimeStamp.ToString (Config.TimeStampFormat) + " " + logItem.Message);
                    }
                    break;
                case LogType.Event:
                    foreach (StreamWriter s in EventStream) {
                        s.WriteLine (logItem.TimeStamp.ToString (Config.TimeStampFormat) + " " + logItem.Message);
                    }
                    break;
            }
        }

        /// <summary>
        /// 
        ///     Runs a single package
        /// 
        /// </summary>
        /// <param name="config"></param>
        /// <param name="package"></param>
        private static void RunPackage (Configuration config, PackageInformation package) {
            try {
                //
                //  Check the log file name
                //
                EventStream.Clear ();
                if (package.EventsToConsole) {
                    EventStream.Add (new StreamWriter (Console.OpenStandardOutput ()));
                }
                if (package.EventsToFileName != "") {
                    EventStream.Add (new StreamWriter (package.EventsToFileName));
                }

                //
                //  Check the events file name
                //
                LogStream.Clear ();
                if (package.LogToConsole) {
                    LogStream.Add (new StreamWriter (Console.OpenStandardOutput ()));
                }
                if (package.LogToFileName != "") {
                    LogStream.Add (new StreamWriter (package.LogToFileName));
                }

                ClientAgent.ClientAgent.ExecutePackage (package, LogMessage);

            } finally {
                foreach (StreamWriter s in LogStream) {
                    s.Close ();
                }
                foreach (StreamWriter s in EventStream) {
                    s.Close ();
                }
            }
        }
    }
}
